/** 
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.servicemix.jbi.config.spring;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.support.BeanDefinitionReader;
import org.springframework.beans.factory.xml.ElementProcessor;
import org.springframework.core.io.Resource;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.servicemix.jbi.config.spring.ElementProcessorSupport;
import org.servicemix.jbi.util.DOMUtil;

import javax.xml.namespace.QName;

/**
 * Handles the 'qname' element
 *
 * @version $Revision: 1.2 $
 */
public class QNameElementProcessor extends ElementProcessorSupport implements ElementProcessor {
    private static final transient Log log = LogFactory.getLog(QNameElementProcessor.class);

    public void processElement(Element element, BeanDefinitionReader beanDefinitionReader, Resource resource) {
        Element root = (Element) element.getParentNode();
        QName qname = createQName(element, DOMUtil.getElementText(element));
        Element bean = addQNameBeanElement(root, qname);

        // now lets discard the old qname element to avoid breaking spring
        root.removeChild(element);

        logXmlGenerated(log, "qname generated", bean);
    }

    /**
     * Adds a new Spring bean element to create a QName instance of the given value.
     */
    protected Element addQNameBeanElement(Element owner, QName qname) {
        Element bean = addBeanElement(owner, "javax.xml.namespace.QName");

        String uri = qname.getNamespaceURI();
        boolean hasURI = uri != null && uri.length() > 0;
        if (hasURI) {
            addConstructorValueNode(bean, uri);
        }
        addConstructorValueNode(bean, qname.getLocalPart());
        if (hasURI) {
            String prefix = qname.getPrefix();
            if (prefix != null && prefix.length() > 0) {
                addConstructorValueNode(bean, prefix);
            }
        }
        return bean;
    }

    /**
     * Creates a QName instance from the given namespace context for the given qualifiedName
     *
     * @param element       the element to use as the namespace context
     * @param qualifiedName the fully qualified name
     * @return the QName which matches the qualifiedName
     */
    protected QName createQName(Element element, String qualifiedName) {
        int index = qualifiedName.indexOf(':');
        if (index >= 0) {
            String prefix = qualifiedName.substring(0, index);
            String localName = qualifiedName.substring(index + 1);
            String uri = findNamespaceURI(element, prefix);
            return new QName(uri, localName, prefix);
        }
        else {
            String uri = findDefaultNamespace(element);
            if (uri != null) {
                return new QName(uri, qualifiedName);
            }
            return new QName(qualifiedName);
        }
    }

    /**
     * Finds the namespace URI mapped to the given prefix on the given element
     */
    protected String findNamespaceURI(Element element, String prefix) {
        NamedNodeMap attributes = element.getAttributes();
        for (int i = 0, size = attributes.getLength(); i < size; i++) {
            Attr attr = (Attr) attributes.item(i);
            String name = attr.getName();
            if (name.startsWith("xmlns:")) {
                String aPrefix = name.substring(6);
                if (prefix.equals(aPrefix)) {
                    return attr.getValue();
                }
            }
        }
        Node parentNode = element.getParentNode();
        if (parentNode instanceof Element) {
            return findNamespaceURI((Element) parentNode, prefix);
        }
        return null;
    }

    /**
     * Finds the default namespace URI on the given element
     */
    protected String findDefaultNamespace(Element element) {
        NamedNodeMap attributes = element.getAttributes();
        for (int i = 0, size = attributes.getLength(); i < size; i++) {
            Attr attr = (Attr) attributes.item(i);
            String name = attr.getName();
            if (name.equals("xmlns")) {
                return attr.getValue();
            }
        }
        Node parentNode = element.getParentNode();
        if (parentNode instanceof Element) {
            return findDefaultNamespace((Element) parentNode);
        }
        return null;
    }

}
