/** 
 * <a href="http://servicemix.org">ServiceMix: The open source ESB</a> 
 * 
 * Copyright 2005 RAJD Consultancy Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.servicemix.jbi.management;
import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.DynamicMBean;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanRegistration;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import org.apache.commons.beanutils.MethodUtils;
import org.apache.commons.beanutils.PropertyUtilsBean;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * An MBean wrapper for an Existing Object
 * 
 * @version $Revision: 1.2 $
 */
public class BaseDynamicMBean implements DynamicMBean, MBeanRegistration {
    private final static Log log = LogFactory.getLog(BaseDynamicMBean.class);
    private Map cachedAttributes = new LinkedHashMap();//used to maintain insertion ordering
    private PropertyUtilsBean beanUtil = new PropertyUtilsBean();
    private MBeanAttributeInfo[] attributeInfos;
    private MBeanOperationInfo[] operationInfos;
    private MBeanInfo beanInfo;
    private Object theBean;
    //this values are set after registering with the MBeanServer//
    private ObjectName objectName;
    private MBeanServer beanServer;

    /**
     * Constructor
     * 
     * @param object
     * @param description
     * @param attrs
     * @param ops
     * @throws ReflectionException
     */
    public BaseDynamicMBean(Object object, String description, MBeanAttributeInfo[] attrs, MBeanOperationInfo[] ops)
            throws ReflectionException {
        this.theBean = object;
        this.attributeInfos = attrs;
        this.operationInfos = ops;
        buildAttributes(this.theBean, this.attributeInfos);
        this.beanInfo = new MBeanInfo(this.theBean.getClass().getName(), description, attrs, null, ops, null);
    }

    /**
     * @return the MBeanINfo for this MBean
     */
    public MBeanInfo getMBeanInfo() {
        return beanInfo;
    }

    /**
     * Retrieve ObjectName of the MBean - set after registration
     * 
     * @return the ObjectName of the MBean
     */
    public ObjectName getObjectName() {
        return objectName;
    }

    /**
     * Retrieve the MBeanServer - set after registration
     * 
     * @return the beanServer
     */
    public MBeanServer getBeanServer() {
        return beanServer;
    }

    /**
     * Get the Value of an Attribute
     * 
     * @param name
     * @return the value of the Attribute
     * @throws AttributeNotFoundException
     * @throws MBeanException
     * @throws ReflectionException
     */
    public Object getAttribute(String name) throws AttributeNotFoundException, MBeanException, ReflectionException {
        Object result = null;
        CachedAttribute ca = (CachedAttribute) cachedAttributes.get(name);
        if (ca != null) {
            try {
                Object value = beanUtil.getProperty(ca.getBean(), name);
            }
            catch (IllegalAccessException e) {
                throw new MBeanException(e);
            }
            catch (InvocationTargetException e) {
                throw new MBeanException(e);
            }
            catch (NoSuchMethodException e) {
                throw new MBeanException(e);
            }
        }
        else {
            throw new AttributeNotFoundException("Could not locate " + name);
        }
        return result;
    }

    /**
     * Set the Attribute
     * 
     * @param attr
     * @throws AttributeNotFoundException
     * @throws InvalidAttributeValueException
     * @throws MBeanException
     * @throws ReflectionException
     */
    public void setAttribute(Attribute attr) throws AttributeNotFoundException, InvalidAttributeValueException,
            MBeanException, ReflectionException {
        String name = attr.getName();
        CachedAttribute ca = (CachedAttribute) cachedAttributes.get(name);
        if (ca != null) {
            try {
                ca.updateAttribute(beanUtil, attr);
            }
            catch (IllegalAccessException e) {
                throw new MBeanException(e);
            }
            catch (InvocationTargetException e) {
                throw new MBeanException(e);
            }
            catch (NoSuchMethodException e) {
                throw new MBeanException(e);
            }
        }
        else {
            throw new AttributeNotFoundException("Could not locate " + name);
        }
    }

    /**
     * @param attributes - array of Attribute names
     * @return AttributeList of matching Attributes
     */
    public AttributeList getAttributes(String[] attributes) {
        AttributeList result = null;
        if (attributes != null) {
            result = new AttributeList();
            for (int i = 0;i < attributes.length;i++) {
                CachedAttribute ca = (CachedAttribute) cachedAttributes.get(attributes[i]);
                result.add(ca.getAttribute());
            }
        }
        else {
            //Do this to maintain insertion ordering
            for (Iterator i = cachedAttributes.entrySet().iterator();i.hasNext();) {
                Map.Entry entry = (Map.Entry) i.next();
                CachedAttribute ca = (CachedAttribute) entry.getValue();
                result.add(ca.getAttribute());
            }
        }
        return result;
    }

    /**
     * Set values of Attributes
     * 
     * @param attributes
     * @return the list of Attributes set with their new values
     */
    public AttributeList setAttributes(AttributeList attributes) {
        AttributeList result = new AttributeList();
        if (attributes != null) {
            for (int i = 0;i < attributes.size();i++) {
                Attribute attribute = (Attribute) attributes.get(i);
                try {
                    setAttribute(attribute);
                }
                catch (AttributeNotFoundException e) {
                    log.warn("Failed to setAttribute(" + attribute + ")", e);
                }
                catch (InvalidAttributeValueException e) {
                    log.warn("Failed to setAttribute(" + attribute + ")", e);
                }
                catch (MBeanException e) {
                    log.warn("Failed to setAttribute(" + attribute + ")", e);
                }
                catch (ReflectionException e) {
                    log.warn("Failed to setAttribute(" + attribute + ")", e);
                }
                result.add(attribute);
            }
        }
        return result;
    }

    /**
     * Invoke an operation
     * 
     * @param name
     * @param params
     * @param signature
     * @return result of invoking an operation
     * @throws MBeanException
     * @throws ReflectionException
     */
    public Object invoke(String name, Object[] params, String[] signature) throws MBeanException, ReflectionException {
        try {
            return MethodUtils.invokeMethod(theBean, name, params);
        }
        catch (NoSuchMethodException e) {
            throw new ReflectionException(e);
        }
        catch (IllegalAccessException e) {
            throw new ReflectionException(e);
        }
        catch (InvocationTargetException e) {
            throw new ReflectionException(e);
        }
    }

    /**
     * Called at registration
     * 
     * @param mbs
     * @param on
     * @return the ObjectName
     * @throws Exception
     */
    public ObjectName preRegister(MBeanServer mbs, ObjectName on) throws Exception {
        this.beanServer = mbs;
        this.objectName = on;
        return on;
    }

    /**
     * Caled post registration
     * 
     * @param done
     */
    public void postRegister(Boolean done) {
    }

    /**
     * Called before removal from the MBeanServer
     * 
     * @throws Exception
     */
    public void preDeregister() throws Exception {
    }

    /**
     * Called after removal from the MBeanServer
     */
    public void postDeregister() {
    }

    /**
     * build internal Map of CachedAttributes
     * 
     * @param obj
     * @param attrs
     * @throws ReflectionException
     */
    private void buildAttributes(Object obj, MBeanAttributeInfo[] attrs) throws ReflectionException {
        if (attrs != null) {
            for (int i = 0;i < attrs.length;i++) {
                try {
                    String name = attrs[i].getName();
                    PropertyDescriptor pd = beanUtil.getPropertyDescriptor(obj, name);
                    Object value = beanUtil.getProperty(obj, name);
                    Attribute attribute = new Attribute(name, value);
                    CachedAttribute ca = new CachedAttribute(attribute);
                    ca.setBean(obj);
                    ca.setPropertyDescriptor(pd);
                    cachedAttributes.put(name, ca);
                }
                catch (NoSuchMethodException e) {
                    throw new ReflectionException(e);
                }
                catch (IllegalAccessException e) {
                    throw new ReflectionException(e);
                }
                catch (InvocationTargetException e) {
                    throw new ReflectionException(e);
                }
            }
        }
    }
}