/** 
 * <a href="http://servicemix.org">ServiceMix: The open source ESB</a> 
 * 
 * Copyright 2005 RAJD Consultancy Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.servicemix.jbi.messaging;

import java.io.Serializable;
import java.net.URI;
import java.util.Set;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.transaction.Transaction;
import javax.xml.namespace.QName;
import org.servicemix.jbi.component.ComponentContextImpl;
import org.servicemix.jbi.container.ActivationSpec;
import org.servicemix.jbi.framework.ComponentNameSpace;
import org.servicemix.jbi.servicedesc.ServiceEndpointImpl;
import org.servicemix.client.Marshaler;
import org.servicemix.client.DefaultMarshaler;

/**
 * A simple message exchange declaration. This is partial, just giving us enough ME function for the doodle. This
 * doesn't add anything new to the current MessageExchange definition.
 *
 * @version $Revision: 1.12 $
 */
public class MessageExchangeImpl implements MessageExchange, Serializable {

    private ComponentContextImpl sourceContext;
    private ExchangeStatus status = ExchangeStatus.ACTIVE;
    private Role role = Role.PROVIDER;
    private ExchangePacket packet;
    private Marshaler marshaler;

    /**
     * Constructor
     * @param exchangeId
     * @param pattern
     */
    public MessageExchangeImpl(String exchangeId, URI pattern) {
        this.packet = new ExchangePacket();
        this.packet.setExchangeId(exchangeId);
        this.packet.setPattern(pattern);
        this.packet.setStatus(this.status.toString());
        this.packet.setOutbound(true);
    }
    
    /**
     * Construct an exchange from an ExchangePacket
     * @param packet
     */
    public MessageExchangeImpl(ExchangePacket packet){
        setPacket(packet);
    }

    /**
     * Returns the activation spec that was provided when the component was registered
     * @return the spec
     */
    public ActivationSpec getActivationSpec() {
        if (sourceContext != null) {
            return sourceContext.getActivationSpec();
        }
        return null;
    }

    /**
     * Returns the context which created the message exchange which can then be used for routing
     * @return the context
     */
    public ComponentContextImpl getSourceContext() {
        return sourceContext;
    }

    /**
     * Set the context
     * @param sourceContext
     */
    public void setSourceContext(ComponentContextImpl sourceContext) {
        this.sourceContext = sourceContext;
    }

    /**
     * Set the packet
     * @param packet
     */
    public void setPacket(ExchangePacket packet){
        this.packet = packet;
        if (this.packet.getRole()==ExchangePacket.CONSUMER_ROLE){
            this.role = Role.PROVIDER;
        }else {
            this.role = Role.CONSUMER;
        }
        this.status = ExchangeStatus.valueOf(packet.getStatus());
    }
    
    /**
     * @return the packet
     */
    public ExchangePacket getPacket(){
        return packet;
    }
    
    /**
     * @return true if an outbound exchange
     */
    public boolean isOutbound(){
        return packet.isOutbound();
    }
    

    /**
     * @return URI of pattenr exchange
     */
    public URI getPattern() {
        return packet.getPattern();
    }

    /**
     * @return the exchange Id
     */
    public String getExchangeId() {
        return packet.getExchangeId();
    }

    /**
     * @return the processing status of the exchange
     */
    public ExchangeStatus getStatus() {
        return status;
    }

    /**
     * set the processing status
     *
     * @param exchangeStatus
     * @throws MessagingException
     */
    public void setStatus(ExchangeStatus exchangeStatus) throws MessagingException {
        this.status = exchangeStatus;
        this.packet.setStatus(this.status.toString());

    }

    /**
     * set the source of a failure
     *
     * @param exception
     */
    public void setError(Exception exception) {
        this.packet.setError(exception);
    }

    /**
     * @return the exception describing a processing error
     */
    public Exception getError() {
        return packet.getError();
    }

    /**
     * @return the fault message for an exchange
     */
    public Fault getFault() {
        return packet.getFault();
    }

    /**
     * set the fault message for the exchange
     *
     * @param fault
     * @throws MessagingException
     */
    public void setFault(Fault fault) throws MessagingException {
        this.packet.setFault(fault);
    }

    /**
     * @return a new message
     * @throws MessagingException
     */
    public NormalizedMessage createMessage() throws MessagingException {
        return new NormalizedMessageImpl(this);
    }

    /**
     * factory method for fault objects
     *
     * @return a new fault
     * @throws MessagingException
     */
    public Fault createFault() throws MessagingException {
        return new FaultImpl();
    }

    /**
     * get a NormalizedMessage based on the message reference
     *
     * @param name
     * @return a NormalizedMessage
     */
    public NormalizedMessage getMessage(String name) {
        return packet.getMessage(name);
    }

    /**
     * set a NormalizedMessage with a named reference
     *
     * @param message
     * @param name
     * @throws MessagingException
     */
    public void setMessage(NormalizedMessage message, String name) throws MessagingException {
        packet.setMessage(message,name);
    }

    /**
     * @param name
     * @return the proerty from the exchange
     */
    public Object getProperty(String name) {
        return packet.getProperty(name);
    }

    /**
     * set a named property on the exchange
     *
     * @param name
     * @param value
     */
    public void setProperty(String name, Object value) {
        packet.setProperty(name, value);
    }
    
    /**
     * @return property names
     */
    public Set getPropertyNames(){
        return packet.getPropertyNames();
    }

    /**
     * Set an endpoint
     *
     * @param endpoint
     */
    public void setEndpoint(ServiceEndpoint endpoint) {
        packet.setEndpoint((ServiceEndpointImpl) endpoint);
    }

    /**
     * set a service
     *
     * @param name
     */
    public void setService(QName name) {
        packet.setServiceName(name);
    }

    /**
     * set an operation
     *
     * @param name
     */
    public void setOperation(QName name) {
        packet.setOperationName(name);
    }
    
    /**
     * set an interface
     *
     * @param name
     */
    public void setInterfaceName(QName name) {
        packet.setInterfaceName(name);
    }

    /**
     * @return the endpoint
     */
    public ServiceEndpoint getEndpoint() {
        return packet.getEndpoint();
    }

    /**
     * @return the service
     */
    public QName getService() {
        return packet.getServiceName();
    }
    
    /**
     * @return the interface name
     */
    public QName getInterfaceName() {
        return packet.getInterfaceName();
    }

    /**
     * @return the operation
     */
    public QName getOperation() {
        return packet.getOperationName();
    }

    /**
     * @return the transaction context
     */
    public Transaction getTransactionContext() {
        return packet.getTransactionContext();
    }

    /**
     * set the transaction
     *
     * @param transaction
     * @throws MessagingException
     */
    public void setTransactionContext(Transaction transaction) throws MessagingException {
        packet.setTransactionContext(transaction);
    }

    /**
     * @return true if transacted
     */
    public boolean isTransacted() {
        return this.packet.getTransactionContext() != null;
    }

    /**
     * @return the Role of this exchange
     */
    public Role getRole() {
        return role;
    }

    /**
     * @return the in message
     */
    public  NormalizedMessage getInMessage() {
        return packet.getInMessage();
    }

    /**
     * set the in message
     *
     * @param message
     * @throws MessagingException
     */
    public  void setInMessage(NormalizedMessage message) throws MessagingException {
        packet.setInMessage(message);
    }

    /**
     * @return the out message
     */
    public  NormalizedMessage getOutMessage() {
        return packet.getOutMessage();
    }

    /**
     * set the out message
     *
     * @param message
     * @throws MessagingException
     */
    public  void setOutMessage(NormalizedMessage message) throws MessagingException {
        packet.setOutMessage(message);
        
    }
    
    /**
     * @return Returns the sourceId.
     */
    public ComponentNameSpace getSourceId() {
        return packet.getSourceId();
    }
    /**
     * @param sourceId The sourceId to set.
     */
    public void setSourceId(ComponentNameSpace sourceId) {
        packet.setSourceId(sourceId);
    }
    
    /**
     * @return Returns the destinationId.
     */
    public ComponentNameSpace getDestinationId() {
        return packet.getDestinationId();
    }
    /**
     * @param destinationId The destinationId to set.
     */
    public void setDestinationId(ComponentNameSpace destinationId) {
        packet.setDestinationId(destinationId);
    }
    
    /**
     * @return Returns the endpointName.
     */
    public String getEndpointName() {
        return packet.getEndpointName();
    }
    /**
     * @param endpointName The endpointName to set.
     */
    public void setEndpointName(String endpointName) {
        packet.setEndpointName(endpointName);
    }


    public synchronized Marshaler getMarshaler() {
        if (marshaler == null) {
            marshaler = new DefaultMarshaler();
        }
        return marshaler;
    }

    public void setMarshaler(Marshaler marshaler) {
        this.marshaler = marshaler;
    }
}