/** 
 * <a href="http://servicemix.org">ServiceMix: The open source ESB</a> 
 * 
 * Copyright 2005 RAJD Consultancy Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.servicemix.jbi.nmr.flow.seda;
import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.management.JMException;
import javax.management.MBeanAttributeInfo;
import javax.management.ObjectName;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.servicemix.jbi.framework.ComponentNameSpace;
import org.servicemix.jbi.management.AttributeInfoHelper;
import org.servicemix.jbi.management.BaseLifeCycle;
import org.servicemix.jbi.messaging.ExchangePacket;
import EDU.oswego.cs.dl.util.concurrent.BoundedLinkedQueue;
import EDU.oswego.cs.dl.util.concurrent.SynchronizedBoolean;

/**
 * A simple Straight through flow
 * 
 * @version $Revision: 1.5 $
 */
public class SedaQueue extends BaseLifeCycle implements Runnable {
    private static final Log log = LogFactory.getLog(SedaQueue.class);
    protected ComponentNameSpace name;
    protected SedaFlow flow;
    protected BoundedLinkedQueue queue;
    protected SynchronizedBoolean started = new SynchronizedBoolean(false);
    protected SynchronizedBoolean running = new SynchronizedBoolean(false);
    protected ObjectName objectName;

    /**
     * SedaQueue name
     * 
     * @param name
     */
    public SedaQueue(ComponentNameSpace name) {
        this.name = name;
    }
    
    /**
     * Get the name
     * @return name
     */
    public String getName(){
        return "SedaQueue." +  name.getName();
    }
    
    /**
     * Get the description
     * @return description
     */
    public String getDescription(){
        return "bounded worker Queue for the NMR";
    }

    /**
     * Initialize the Region
     * 
     * @param flow
     * @param capacity
     */
    public void init(SedaFlow flow, int capacity) {
        this.flow = flow;
        this.queue = new BoundedLinkedQueue(capacity);
    }
    
    /**
     * Set the capacity of the Queue
     * @param value
     */
    public void setCapacity(int value){
        int oldValue = queue.capacity();
        this.queue.setCapacity(value);
        super.firePropertyChanged("capacity", new Integer(oldValue), new Integer(value));
    }
    
    /**
     * @return the capacity of the Queue
     */
    public int getCapacity(){
        return this.queue.capacity();
    }
    
    /**
     * @return size of the Queue
     */
    public int getSize(){
        return this.queue.size();
    }

    /**
     * Enqueue a Packet for processing
     * 
     * @param packet
     * @throws InterruptedException
     */
    public void enqueue(ExchangePacket packet) throws InterruptedException {
        queue.put(packet);
    }

    /**
     * start processing
     * @throws JBIException
     */
    public void start() throws JBIException {
        started.set(true);
        running.set(true);
        synchronized(started){
            started.notify();
        }
        super.start();
    }

    /**
     * stop processing
     * @throws JBIException
     */
    public void stop() throws JBIException {
        started.set(false);
        super.stop();
    }
    
    /**
     * shutDown the Queue
     * @throws JBIException
     *
     */
    public void shutDown() throws JBIException{
        running.set(false);
        super.shutDown();
    }

    /**
     * do processing
     */
    public void run() {
        while (running.get()) {
            ExchangePacket packet = null;
            try {
                packet = (ExchangePacket) queue.poll(1000);
                synchronized(started){
                    while(running.get() && !started.get()){
                        started.wait(500);
                    }
                }
            }
            catch (InterruptedException e) {
                log.warn(this + " interrupted", e);
            }
            if (packet != null) {
                try {
                    flow.doRouting(packet);
                    if (log.isDebugEnabled()) {
                        log.debug(this + " enqueued packet: " + packet);
                    }
                }
                catch (MessagingException e) {
                    log.error(this + " Got messaging exception processing " + packet, e);
                }
            }
        }
    }

    /**
     * @return pretty print
     */
    public String toString() {
        return "SedaQueue{" + name + "}";
    }
    
    /**
     * Get an array of MBeanAttributeInfo
     * 
     * @return array of AttributeInfos
     * @throws JMException
     */
    public MBeanAttributeInfo[] getAttributeInfos() throws JMException {
        AttributeInfoHelper helper = new AttributeInfoHelper();
        helper.addAttribute(getObjectToManage(), "capacity", "The capacity of the SedaQueue");
        helper.addAttribute(getObjectToManage(), "size", "The size (depth) of the SedaQueue");
        return AttributeInfoHelper.join(super.getAttributeInfos(), helper.getAttributeInfos());
    }
    /**
     * @return Returns the objectName.
     */
    public ObjectName getObjectName() {
        return objectName;
    }
    /**
     * @param objectName The objectName to set.
     */
    public void setObjectName(ObjectName objectName) {
        this.objectName = objectName;
    }
}