/*
 * Copyright 2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.servicemix.jbi.servicedesc;

import java.io.Serializable;
import java.util.List;
import java.util.Map;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import org.servicemix.jbi.framework.ComponentNameSpace;
import org.w3c.dom.DocumentFragment;
import EDU.oswego.cs.dl.util.concurrent.ConcurrentHashMap;
import EDU.oswego.cs.dl.util.concurrent.CopyOnWriteArrayList;

/**
 * Reference generated by NMR ro refer to an endpoint registration
 *
 * @version $Revision: 1.6 $
 */
public class ServiceEndpointImpl implements ServiceEndpoint, Serializable {
    private static final long serialVersionUID = -2710298087712302015L;
    private ComponentNameSpace componentName;
    private String endpointName;
    private QName serviceName;
    private Map referenceMap = new ConcurrentHashMap();
    private List interfaces = new CopyOnWriteArrayList();
    
    

    /**
     * Constructor
     * @param ComponentName
     * @param endpointName
     * @param serviceName
     */
    public ServiceEndpointImpl(ComponentNameSpace ComponentName,String endpointName, QName serviceName) {
        this.componentName = ComponentName;
        this.endpointName = endpointName;
        this.serviceName = serviceName;
    }
    
    
    /**
     * get the id of the ComponentConnector
     * @return the id
     */
    public ComponentNameSpace getComponentNameSpace(){
        return componentName;
    }

    /**
     * Get a reference to this endpoint, using an endpoint reference vocabulary
     * that is known to the provider.
     * @param operationName the name of the operation to be performed by a
     * consumer of the generated endpoint reference. Set to <code>null</code>
     * if this is not applicable.
     * @return endpoint reference as an XML fragment; <code>null</code> if the
     * provider does not support such references.
     */
    public DocumentFragment getAsReference(QName operationName){
        return (DocumentFragment)referenceMap.get(operationName);
    }
    
    /**
     * associate a DocumentFragment with an operationName
     * @param operationName
     * @param fragment
     */
    public void setAsReference(QName operationName, DocumentFragment fragment){
        referenceMap.put(operationName,fragment);
    }

    /**
     * Returns the name of this endpoint.
     * @return the endpoint name.
     */
    public String getEndpointName(){
        return endpointName;
    }
    
    /**
     * set the endpoint name
     * @param name
     */
    public void setEndpointName(String name){
        this.endpointName = name;
    }

    /**
     * Get the qualified names of all the interfaces implemented by this
     * service endpoint.
     * @return array of all interfaces implemented by this service endpoint;
     * must be non-null and non-empty.
     */
    public synchronized QName[] getInterfaces(){
        QName[] result = new QName[interfaces.size()];
        interfaces.toArray(result);
        return result;
    }
    
    /**
     * Add an interface
     * @param name
     */
    public synchronized void addInterface(QName name){
        interfaces.add(name);
    }

    /**
     *  Returns the service name of this endpoint.
     *  @return the qualified service name.
     */
    public QName getServiceName(){
       return serviceName; 
    }
    
    /**
     * Set the service Name
     * @param name
     */
    public void setServiceName(QName name){
        this.serviceName = name;
    }
    
    /**
     * @param obj
     * @return true if equal
     */
    public boolean equals(Object obj){
        boolean result = false;
        if (obj != null && obj instanceof ServiceEndpointImpl){
            ServiceEndpointImpl other = (ServiceEndpointImpl)obj;
            result = other.componentName.equals(this.componentName) && other.endpointName.equals(this.endpointName);
        }
        return result;
    }
    
   
    /**
     * @return has code
     */
    public int hashCode(){
        return componentName.hashCode() ^ endpointName.hashCode();
    }
    
    /**
     * @return a pretty print of this
     */
    public String toString() {
        return "ServiceEndpoint[service: " + serviceName + "; endpoint: " + endpointName + "]";
    }
}